<?php

namespace Ardent\Undercurrent\Http;

use Ardent\Undercurrent\Container\ClassNotFoundException;
use Ardent\Undercurrent\Container\ContainerInterface;
use Exception;
use ReflectionMethod;

class GenericRouter implements RouterInterface
{
    public function __construct(
        private readonly ContainerInterface $container,
        private readonly RouterConfig       $config,
    )
    {
    }

    /**
     * @throws RouteNotFoundException|ClassNotFoundException
     */
    public function dispatch(RequestInterface $request): ResponseInterface
    {
        foreach ($this->config->getRoutes() as $route) {
            $params = $this->resolveParams($route->getRoute()->path, $request->getUri());

            if ($params === null) {
                continue;
            }

            $controller = $this->container->get($route->getController());
            $method = $route->getMethod();
//            $params = $this->autowire($route) + $this->resolveParams($route->getRoute()->path, $request->getUri());
            $params = $this->resolveParams($route->getRoute()->path, $request->getUri());
            return $controller->$method(...$params);
        }

        throw new RouteNotFoundException($request);
    }

    private function resolveParams(string $routeUri, string $requestUri): ?array
    {
        $result = preg_match_all('/{(\w+)}/', $routeUri, $tokens);

        if (!$result) {
            return null;
        }

        $tokens = $tokens[0];

        $matchingRegex = '/^' . str_replace(
                ['/', ...$tokens],
                ['\/', ...array_fill(0, count($tokens), '([\w\s]+)')],
                $routeUri
            ) . '$/';

        $result = preg_match($matchingRegex, $requestUri, $matches);
        if ($result === 0) {
            return [];
        }
        $matches = array_slice($matches, 1);

        return array_combine(array_map(fn($token) => trim($token, '{}'), $tokens), $matches);
    }

    private function autowire(RouteConfig $route): array
    {
        $reflectionMethod = new ReflectionMethod($route->getController(), $route->getMethod());

        $params = [];
        foreach ($reflectionMethod->getParameters() as $parameter) {
            $type = $parameter->getType();
            if (!$type) {
                throw new Exception('Cannot autowire parameter without type');
            }
            $params[] = $this->container->get($type->getName());
        }

        return $params;
    }
}